package manger

import (
	"encoding/json"
	"github.com/link33/sidecar/internal/port"
	"github.com/link33/sidecar/internal/router"
	"github.com/link33/sidecar/model/pb"

	"github.com/link33/sidecar/internal/peermgr"
	appchainmgr "github.com/meshplus/bitxhub-core/appchain-mgr"
	"github.com/sirupsen/logrus"
)

type Manager struct {
	logger logrus.FieldLogger
	Mgr    appchainmgr.AppchainMgr
	pm     peermgr.PeerManager
	router router.Router
}

func (mgr *Manager) Start() error {
	panic("implement me")
}

func (mgr *Manager) Stop() error {
	panic("implement me")
}

func NewManager(addr string, router router.Router, pm peermgr.PeerManager, Mgr appchainmgr.AppchainMgr, logger logrus.FieldLogger) (*Manager, error) {
	am := &Manager{
		Mgr:    Mgr,
		logger: logger,
		router: router,
		pm:     pm,
	}

	err := pm.RegisterMultiMsgHandler([]pb.Message_Type{
		pb.Message_APPCHAIN_REGISTER,
		pb.Message_APPCHAIN_UPDATE,
		pb.Message_APPCHAIN_GET,
	}, am.handleMessage)
	if err != nil {
		return nil, err
	}

	return am, nil
}

func (mgr *Manager) handleMessage(s port.Port, msg *pb.Message) {
	var res []byte
	var ok bool
	switch msg.Type {
	case pb.Message_APPCHAIN_REGISTER:
		ok, res = mgr.Mgr.Register(msg.Payload.Data)
	case pb.Message_APPCHAIN_UPDATE:
		ok, res = mgr.Mgr.Update(msg.Payload.Data)
	case pb.Message_APPCHAIN_GET:
		app := &appchainmgr.Appchain{}
		if err := json.Unmarshal(msg.Payload.Data, app); err != nil {
			mgr.logger.Error(err)
			return
		}
		ok, res = mgr.Mgr.QueryById(app.ID, nil)
	default:
		m := "wrong appchain message type"
		res = []byte(m)
		mgr.logger.Error(m)
	}

	ackMsg := pb.Msg(msg.Type, ok, res)
	err := s.AsyncSend(ackMsg)
	if err != nil {
		mgr.logger.Error(err)
	}

	appchainRes := &appchainmgr.Appchain{}
	if err := json.Unmarshal(res, appchainRes); err != nil {
		mgr.logger.Error(err)
		return
	}

	mgr.logger.WithFields(logrus.Fields{
		"type":           msg.Type,
		"from_id":        appchainRes.ID,
		"name":           appchainRes.Name,
		"desc":           appchainRes.Desc,
		"chain_type":     appchainRes.ChainType,
		"consensus_type": appchainRes.ConsensusType,
	}).Info("Handle appchain message")
}
