package router

import (
	"github.com/link33/sidecar/internal/port"
	"github.com/link33/sidecar/model/pb"
)

//go:generate mockgen -destination mock_router/mock_router.go -package mock_router -source interface.go
type Router interface {
	// Start starts the router module
	Start() error

	// Stop
	Stop() error

	Add(p port.Port) error

	Adds(p []port.Port) error

	//Route sends ibtp to the union pier in target relay chain
	Route(ibtp *pb.Message) error

	Load(key string) (value port.Port, ok bool)

	//InPut(ibtp *pb.IBTP) chan *pb.IBTP
	//OutPut(ibtp *pb.IBTP) chan *pb.IBTP
	//代替上面两个方法
	//Router(ibtp *pb.IBTP) chan *pb.IBTP
}

//路由规则，根据路由表，rules接口， 这个放到交易内部。可以让用户决定。并且签名。
// 1、广播
// 2、单一路由，指定路由标识。
// 3、p2p模式的路由，
// 4、中继路由
// 5、定制化：

//验证规则
//1、收集多个节点
//2、收集指定节点
//3、收集共识节点
// 即满足多个条件

//路由规则优先级：
//1、用户交易内部的路由规则最高。
//2、用户在程序设定。
//3、系统默认。
